==> daemonshogi-0.1.3.tar.gz: daemonshogi-0.1.3/src/si/minmax.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * mixmax.c
 * $Source: /cvs/si/minmax.c,v $
 * $Id: minmax.c,v 1.44 2002/09/17 20:00:20 mik Exp $
 */

#include <stdio.h>
#include <assert.h>
#include "si.h"
#include "ui.h"

#ifdef DEBUG
static long max_point;
static long min_point;
#endif /* DEBUG */

/* minmax.c ǤϥϥåȤʤ */
#undef USE_HASH

int jouseki(BOARD *bo, TE *te);

/**
 * min/max õԤ
 * @param bo оݤBOARD
 * @return õ̤֤
 *         SI_NORMALʤ̾μꡣSI_WINʤХץξ
 *         SI_LOSTʤХץ餱
 */
INPUTSTATUS minmax(BOARD *bo) {
  int alpha, beta;
  int i;
  static TREE root;
  static MOVEINFO mi;
  int king_xy;
  int num_of_ohte;
  extern TE best_te;
  extern int MATE_DEEP_MAX;
  extern GAME g_game;
#ifdef DEBUG
  extern long G_COUNT_ALPHA_CUT;
  extern long G_COUNT_BETA_CUT;
  extern long G_F_MIN_PUT_HASH;
  extern long G_F_MAX_PUT_HASH;
#endif /* DEBUG */

#ifdef DEBUG
  G_COUNT_ALPHA_CUT = 0;
  G_COUNT_BETA_CUT = 0;
  G_F_MIN_PUT_HASH = 0;
  G_F_MAX_PUT_HASH = 0;
#endif /* DEBUG */

#ifdef NOREAD   
  if ( jouseki(bo, &best_te) ) {
    /* ׼꤬ä */
     
    return SI_NORMAL;
  }
#endif /* NOREAD */
   
  king_xy = bo->code_xy[ 1 + (bo->next == 0) ][ 0 ];

  best_te.fm = 0;
  best_te.to = 0;
  best_te.nari = 0;
  best_te.uti = 0;
  
  moveto2(bo, &mi, king_xy, bo->next);

  if ( 0 < mi.count ) {
    /* ꤵ줿֤ǻ׹ͥ롼ä */
    return SI_WIN;
  }
  
  alpha = -30000;
  beta = 30000;
  
  king_xy = bo->code_xy[1 + bo->next][0];
  /* ꤵƤ뤫Ĵ٤ */
  num_of_ohte = boGetToBoardCount(king_xy, (bo->next == 0));

  if ( num_of_ohte == 0 ) {
    /* ̤ꤵƤʤäͤߤ뤫Ĵ٤ */
    initTREE( &root );

    for ( i=0; i<6; i++ ) {
      MATE_DEEP_MAX = i;
      if ( mate(bo, &root, 0) ) {
	/* ͤ */
	break;
      }
    }
    
    if ( root.child != NULL ) {
      /* check mate */
      best_te.fm = root.child->te.fm;
      best_te.to = root.child->te.to;
      best_te.nari = root.child->te.nari;
      best_te.uti = root.child->te.uti;
    
      freeTREE( root.child );
      return SI_NORMAL;
    }
  }
  
#ifdef DEBUG
  max_point = -3000000;
  min_point = 3000000;
#endif /* DEBUG */
  
  f_max( bo, 0, alpha, beta );

  if ( gmGetGameStatus(&g_game) == SI_CHUDAN ) {
    return SI_CHUDAN_RECV;
  } 

  if ( best_te.fm == 0 && best_te.to == 0 && best_te.nari == 0 &&
       best_te.uti == 0 ) {
    /* ؤ꤬ʤ餱 */
    return SI_TORYO;
  }

  return SI_NORMAL;
}

/**
 * minؿ
 * ζ̤õ롣ɾʤ褦˼֡
 * @param bo оݤBOARD
 * @param depth ο
 * @param alpha alpha
 * @param beta beta
 * @return
 */
int f_min(BOARD *bo, int depth, int alpha, int beta) {
  MOVEINFO *mi;
  int point, value;
  int flg_tori;
  int prev_beta;
  int i;
  extern int THINK_DEEP_MAX;
  extern long count_minmax;
  extern GAME g_game;
  
#ifdef DEBUG
  extern long G_COUNT_BETA_CUT;
#endif /* DEBUG */

  count_minmax++;

#ifdef USE_GUI  
  if ( depth < 2 ) {
    /* ǤλäƤʤĴ٤ */
    if ( gmGetGameStatus(&g_game) == SI_CHUDAN ||
	 gmGetGameStatus(&g_game) == SI_TORYO ||
	 gmGetChudanFlg(&g_game) == 1) {
      /* ǤޤλäΤǽλ */
      return -30000;
    }
    /* GUIΥڥǥ󥰤줿٥Ȥ */
    processing_pending();
  }
#endif /* USE_GUI */
  

#ifdef DEBUG
  assert( bo != NULL );
#endif /* DEBUG */

  if ( depth >= THINK_DEEP_MAX ) {
    return bo->point[ (bo->next == 0) ] - bo->point[ bo->next ];
  }

#ifdef USE_HASH
  /* ϥå̤ϿƤʤĴ٤ */
  if ( hsGetBOARDMinMax(bo, &point, bo->mi.count - depth, 0 ) ) {
    /* ϿƤ */
    return point;
  }
#endif /* USE_HASH */

  point = 99999999;

  mi = newMOVEINFO();

  think( bo, mi, depth );

  prev_beta = beta;

  if ( mi->count == 0 ) {
    /* 롼ؤ꤬ʤԤ */
    freeMOVEINFO( mi );

    return -30000;
  }

  for (i=0; i<mi->count; i++) {
    flg_tori = boMove( bo, &(mi->te[ i ]), bo->next);
    depth++;
    boToggleNext( bo );
    /*
    if ( flg_tori && 10 <= THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori && depth != THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori )
      THINK_DEEP_MAX += 2;
    */
    value = f_max(bo, depth, alpha, beta);
    /*
    if ( flg_tori )
      THINK_DEEP_MAX -= 2;
    */
    boToggleNext( bo );
    boBack(bo, bo->next);
    depth--;

    /*
    if ( value < beta ) {
      beta = value;
    }
    if ( beta <= alpha ) {
      freeMOVEINFO( mi );
      return beta;
    }
    */

#define NOREAD2
#ifdef NOREAD2
    if ( value < point ) {
      point = value;
      if ( point < beta )
	beta = point;

      /* ޴ */
      if ( alpha >= beta ) {
#ifdef USE_HASH
	/* ǥϥåض̤Ͽ */
	hsPutBOARDMinMax(bo, value, bo->mi.count - depth, 0);
#endif /* USE_HASH */
#ifdef DEBUG
	G_COUNT_BETA_CUT++;
#endif /* DEBUG */
	freeMOVEINFO( mi );
	return beta;
      }
    }
#endif /* NOREAD2 */

  }
  
  freeMOVEINFO( mi );

  if ( prev_beta != beta ) {
#ifdef USE_HASH
    /* Ǥ beta ͤ줿ϥåϿ */
    hsPutBOARDMinMax(bo, point, bo->mi.count - depth, 0);
#endif /* USE_HASH */
  }

  return point;
}

/**
 * maxؿ
 * ʬζ̤õ롣ɾ礭ʤ褦˼֡
 * @param bo оݤBOARD
 * @param depth ο
 * @param alpha alpha
 * @param beta beta
 * @return
 */
int f_max(BOARD *bo, int depth, int alpha, int beta) {
  MOVEINFO *mi;
  int point, value;
  int prev_alpha;
  int i;
  extern int THINK_DEEP_MAX;
  extern TE best_te;
  extern long count_minmax;
  int flg_tori;
#ifdef DEBUG
  int dpoint;
  extern long G_COUNT_ALPHA_CUT;
#endif /* DEBUG */
  
  count_minmax++;

  if ( depth >= THINK_DEEP_MAX ) {
    return bo->point[ bo->next ] - bo->point[ (bo->next == 0)];
  }

  /* ϥå̤ϿƤʤĴ٤ */
#ifdef USE_HASH
  if ( hsGetBOARDMinMax(bo, &point, bo->mi.count - depth, 0 ) ) {
    /* ϿƤ */
    return point;
  }
#endif /* USE_HASH */

  point = -99999999;

  mi = newMOVEINFO();
  mi->count = 0;

  think( bo, mi, depth );
  
  if ( mi->count == 0 ) {
    /* 롼ؤ꤬ʤԤ */
    freeMOVEINFO( mi );

    return -30000;
  }

  if ( depth == 0 ) {
    best_te = mi->te[ 0 ];
  }

  prev_alpha = alpha;

  for (i=0; i<mi->count; i++) {
#ifdef DEBUG
    dpoint = bo->point[ 0 ];
#endif /* DEBUG */
    /*
    for ( j=0; j<depth; j++ )
      putchar(' ');
    printTE( &(mi->te[ i ]) );
     */ 
    flg_tori = boMove( bo, &(mi->te[ i ]), bo->next);
    depth++;
    boToggleNext( bo );
    /*
    if ( flg_tori && 10 <= THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori && depth != THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori )
      THINK_DEEP_MAX += 2;
    */
    value = f_min(bo, depth, alpha, beta);
    /*
    if ( flg_tori )
      THINK_DEEP_MAX -= 2;
    */
    boToggleNext( bo );
    boBack(bo, bo->next);
    depth--;

    /*
    if ( alpha < value ) {
      alpha = value;
      if ( depth == 0 ) {
	best_te = mi->te[ i ];
      }
    }
    if ( beta <= alpha ) {
      freeMOVEINFO( mi );
      return alpha;
    }
     */

    if ( point < value ) {
      point = value;
      if ( alpha < point ) {
	alpha = point;
      }

      /* ޴ */
      if ( alpha >= beta ) {
#ifdef USE_HASH
	/* ǥϥåض̤Ͽ */
	hsPutBOARDMinMax(bo, value, bo->mi.count - depth, 0);
#endif /* USE_HASH */
#ifdef DEBUG
	G_COUNT_ALPHA_CUT++;
#endif /* DEBUG */
	/* printf("  alpha cut (alpha,beta)=(%d,%d)\n", alpha, beta); */
	freeMOVEINFO( mi );
	return alpha;
      }

      if ( depth == 0 ) {
#ifdef DEBUG      
	printf("hit - %d - ", point);
	printTE( &(mi->te[ i ]) );
#endif /* DEBUG */
	best_te = mi->te[ i ];
      }
    }

#ifdef DEBUG    
    assert( dpoint == bo->point[ 0 ] );
#endif /* DEBUG */
  }

  freeMOVEINFO( mi );

  if ( prev_alpha != alpha ) {
#ifdef USE_HASH
    /* Ǥ alphͤ줿ϥåϿ */
    hsPutBOARDMinMax(bo, point, bo->mi.count - depth, 0);
#endif /* USE_HASH */
  }

  /* return alpha; */
  return point;
}

/**
 * ׼꤬뤫Ĵ٤Ƥä te ˳Ǽ롣
 * @param bo оݤBOARD
 * @return 1 ʤ׼꤬ä 0 ʤФʤ
 */
int jouseki(BOARD *bo, TE *te) {
  if ( bo->next == SENTE ) {
    if ( bo->mi.count == 0 ) {
      te->fm = 0x77;
      te->to = 0x67;
      te->nari = 0;
      te->uti = 0;
      return 1;
    } else if ( bo->mi.count == 2 ) {
      te->fm = 0x96;
      te->to = 0x87;
      te->nari = 0;
      te->uti = 0;
      return 1;
    }
  } else {
    if ( bo->mi.count == 1 ) {
      te->fm = 0x14;
      te->to = 0x23;
      te->nari = 0;
      te->uti = 0;
      return 1;
    } else if ( bo->mi.count == 3 ) {
      te->fm = 0x33;
      te->to = 0x43;
      te->nari = 0;
      te->uti = 0;
      return 1;
    }
  }
   
  return 0;
}

==> daemonshogi-0.1.4.tar.gz: daemonshogi-0.1.4/src/si/minmax.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * mixmax.c
 * $Source: /cvsroot/daemonshogi/daemonshogi/src/si/minmax.c,v $
 * $Id: minmax.c,v 1.1.1.1 2005/12/09 09:03:09 tokita Exp $
 */

#include <stdio.h>
#include <assert.h>
#include "si.h"
#include "ui.h"

#ifdef DEBUG
static long max_point;
static long min_point;
#endif /* DEBUG */

/* minmax.c ǤϥϥåȤʤ */
#undef USE_HASH

int jouseki(BOARD *bo, TE *te);

/**
 * min/max õԤ
 * @param bo оݤBOARD
 * @return õ̤֤
 *         SI_NORMALʤ̾μꡣSI_WINʤХץξ
 *         SI_LOSTʤХץ餱
 */
INPUTSTATUS minmax(BOARD *bo) {
  int alpha, beta;
  int i;
  static TREE root;
  static MOVEINFO mi;
  int king_xy;
  int num_of_ohte;
  extern TE best_te;
  extern int MATE_DEEP_MAX;
  extern GAME g_game;
#ifdef DEBUG
  extern long G_COUNT_ALPHA_CUT;
  extern long G_COUNT_BETA_CUT;
  extern long G_F_MIN_PUT_HASH;
  extern long G_F_MAX_PUT_HASH;
#endif /* DEBUG */

#ifdef DEBUG
  G_COUNT_ALPHA_CUT = 0;
  G_COUNT_BETA_CUT = 0;
  G_F_MIN_PUT_HASH = 0;
  G_F_MAX_PUT_HASH = 0;
#endif /* DEBUG */

#ifdef NOREAD   
  if ( jouseki(bo, &best_te) ) {
    /* ׼꤬ä */
     
    return SI_NORMAL;
  }
#endif /* NOREAD */
   
  king_xy = bo->code_xy[ 1 + (bo->next == 0) ][ 0 ];

  best_te.fm = 0;
  best_te.to = 0;
  best_te.nari = 0;
  best_te.uti = 0;
  
  moveto2(bo, &mi, king_xy, bo->next);

  if ( 0 < mi.count ) {
    /* ꤵ줿֤ǻ׹ͥ롼ä */
    return SI_WIN;
  }
  
  alpha = -30000;
  beta = 30000;
  
  king_xy = bo->code_xy[1 + bo->next][0];
  /* ꤵƤ뤫Ĵ٤ */
  num_of_ohte = boGetToBoardCount(king_xy, (bo->next == 0));

  if ( num_of_ohte == 0 ) {
    /* ̤ꤵƤʤäͤߤ뤫Ĵ٤ */
    initTREE( &root );

    for ( i=0; i<6; i++ ) {
      MATE_DEEP_MAX = i;
      if ( mate(bo, &root, 0) ) {
	/* ͤ */
	break;
      }
    }
    
    if ( root.child != NULL ) {
      /* check mate */
      best_te.fm = root.child->te.fm;
      best_te.to = root.child->te.to;
      best_te.nari = root.child->te.nari;
      best_te.uti = root.child->te.uti;
    
      freeTREE( root.child );
      return SI_NORMAL;
    }
  }
  
#ifdef DEBUG
  max_point = -3000000;
  min_point = 3000000;
#endif /* DEBUG */
  
  f_max( bo, 0, alpha, beta );

  if ( gmGetGameStatus(&g_game) == SI_CHUDAN ) {
    return SI_CHUDAN_RECV;
  } 

  if ( best_te.fm == 0 && best_te.to == 0 && best_te.nari == 0 &&
       best_te.uti == 0 ) {
    /* ؤ꤬ʤ餱 */
    return SI_TORYO;
  }

  return SI_NORMAL;
}

/**
 * minؿ
 * ζ̤õ롣ɾʤ褦˼֡
 * @param bo оݤBOARD
 * @param depth ο
 * @param alpha alpha
 * @param beta beta
 * @return
 */
int f_min(BOARD *bo, int depth, int alpha, int beta) {
  MOVEINFO *mi;
  int point, value;
  int flg_tori;
  int prev_beta;
  int i;
  extern int THINK_DEEP_MAX;
  extern long count_minmax;
  extern GAME g_game;
  
#ifdef DEBUG
  extern long G_COUNT_BETA_CUT;
#endif /* DEBUG */

  count_minmax++;

#ifdef USE_GUI  
  if ( depth < 2 ) {
    /* ǤλäƤʤĴ٤ */
    if ( gmGetGameStatus(&g_game) == SI_CHUDAN ||
	 gmGetGameStatus(&g_game) == SI_TORYO ||
	 gmGetChudanFlg(&g_game) == 1) {
      /* ǤޤλäΤǽλ */
      return -30000;
    }
    /* GUIΥڥǥ󥰤줿٥Ȥ */
    processing_pending();
  }
#endif /* USE_GUI */
  

#ifdef DEBUG
  assert( bo != NULL );
#endif /* DEBUG */

  if ( depth >= THINK_DEEP_MAX ) {
    return bo->point[ (bo->next == 0) ] - bo->point[ bo->next ];
  }

#ifdef USE_HASH
  /* ϥå̤ϿƤʤĴ٤ */
  if ( hsGetBOARDMinMax(bo, &point, bo->mi.count - depth, 0 ) ) {
    /* ϿƤ */
    return point;
  }
#endif /* USE_HASH */

  point = 99999999;

  mi = newMOVEINFO();

  think( bo, mi, depth );

  prev_beta = beta;

  if ( mi->count == 0 ) {
    /* 롼ؤ꤬ʤԤ */
    freeMOVEINFO( mi );

    return -30000;
  }

  for (i=0; i<mi->count; i++) {
    flg_tori = boMove( bo, &(mi->te[ i ]), bo->next);
    depth++;
    boToggleNext( bo );
    /*
    if ( flg_tori && 10 <= THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori && depth != THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori )
      THINK_DEEP_MAX += 2;
    */
    value = f_max(bo, depth, alpha, beta);
    /*
    if ( flg_tori )
      THINK_DEEP_MAX -= 2;
    */
    boToggleNext( bo );
    boBack(bo, bo->next);
    depth--;

    /*
    if ( value < beta ) {
      beta = value;
    }
    if ( beta <= alpha ) {
      freeMOVEINFO( mi );
      return beta;
    }
    */

#define NOREAD2
#ifdef NOREAD2
    if ( value < point ) {
      point = value;
      if ( point < beta )
	beta = point;

      /* ޴ */
      if ( alpha >= beta ) {
#ifdef USE_HASH
	/* ǥϥåض̤Ͽ */
	hsPutBOARDMinMax(bo, value, bo->mi.count - depth, 0);
#endif /* USE_HASH */
#ifdef DEBUG
	G_COUNT_BETA_CUT++;
#endif /* DEBUG */
	freeMOVEINFO( mi );
	return beta;
      }
    }
#endif /* NOREAD2 */

  }
  
  freeMOVEINFO( mi );

  if ( prev_beta != beta ) {
#ifdef USE_HASH
    /* Ǥ beta ͤ줿ϥåϿ */
    hsPutBOARDMinMax(bo, point, bo->mi.count - depth, 0);
#endif /* USE_HASH */
  }

  return point;
}

/**
 * maxؿ
 * ʬζ̤õ롣ɾ礭ʤ褦˼֡
 * @param bo оݤBOARD
 * @param depth ο
 * @param alpha alpha
 * @param beta beta
 * @return
 */
int f_max(BOARD *bo, int depth, int alpha, int beta) {
  MOVEINFO *mi;
  int point, value;
  int prev_alpha;
  int i;
  extern int THINK_DEEP_MAX;
  extern TE best_te;
  extern long count_minmax;
  int flg_tori;
#ifdef DEBUG
  int dpoint;
  extern long G_COUNT_ALPHA_CUT;
#endif /* DEBUG */
  
  count_minmax++;

  if ( depth >= THINK_DEEP_MAX ) {
    return bo->point[ bo->next ] - bo->point[ (bo->next == 0)];
  }

  /* ϥå̤ϿƤʤĴ٤ */
#ifdef USE_HASH
  if ( hsGetBOARDMinMax(bo, &point, bo->mi.count - depth, 0 ) ) {
    /* ϿƤ */
    return point;
  }
#endif /* USE_HASH */

  point = -99999999;

  mi = newMOVEINFO();
  mi->count = 0;

  think( bo, mi, depth );
  
  if ( mi->count == 0 ) {
    /* 롼ؤ꤬ʤԤ */
    freeMOVEINFO( mi );

    return -30000;
  }

  if ( depth == 0 ) {
    best_te = mi->te[ 0 ];
  }

  prev_alpha = alpha;

  for (i=0; i<mi->count; i++) {
#ifdef DEBUG
    dpoint = bo->point[ 0 ];
#endif /* DEBUG */
    /*
    for ( j=0; j<depth; j++ )
      putchar(' ');
    printTE( &(mi->te[ i ]) );
     */ 
    flg_tori = boMove( bo, &(mi->te[ i ]), bo->next);
    depth++;
    boToggleNext( bo );
    /*
    if ( flg_tori && 10 <= THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori && depth != THINK_DEEP_MAX )
      flg_tori = 0;
    if ( flg_tori )
      THINK_DEEP_MAX += 2;
    */
    value = f_min(bo, depth, alpha, beta);
    /*
    if ( flg_tori )
      THINK_DEEP_MAX -= 2;
    */
    boToggleNext( bo );
    boBack(bo, bo->next);
    depth--;

    /*
    if ( alpha < value ) {
      alpha = value;
      if ( depth == 0 ) {
	best_te = mi->te[ i ];
      }
    }
    if ( beta <= alpha ) {
      freeMOVEINFO( mi );
      return alpha;
    }
     */

    if ( point < value ) {
      point = value;
      if ( alpha < point ) {
	alpha = point;
      }

      /* ޴ */
      if ( alpha >= beta ) {
#ifdef USE_HASH
	/* ǥϥåض̤Ͽ */
	hsPutBOARDMinMax(bo, value, bo->mi.count - depth, 0);
#endif /* USE_HASH */
#ifdef DEBUG
	G_COUNT_ALPHA_CUT++;
#endif /* DEBUG */
	/* printf("  alpha cut (alpha,beta)=(%d,%d)\n", alpha, beta); */
	freeMOVEINFO( mi );
	return alpha;
      }

      if ( depth == 0 ) {
#ifdef DEBUG      
	printf("hit - %d - ", point);
	printTE( &(mi->te[ i ]) );
#endif /* DEBUG */
	best_te = mi->te[ i ];
      }
    }

#ifdef DEBUG    
    assert( dpoint == bo->point[ 0 ] );
#endif /* DEBUG */
  }

  freeMOVEINFO( mi );

  if ( prev_alpha != alpha ) {
#ifdef USE_HASH
    /* Ǥ alphͤ줿ϥåϿ */
    hsPutBOARDMinMax(bo, point, bo->mi.count - depth, 0);
#endif /* USE_HASH */
  }

  /* return alpha; */
  return point;
}

/**
 * ׼꤬뤫Ĵ٤Ƥä te ˳Ǽ롣
 * @param bo оݤBOARD
 * @return 1 ʤ׼꤬ä 0 ʤФʤ
 */
int jouseki(BOARD *bo, TE *te) {
  if ( bo->next == SENTE ) {
    if ( bo->mi.count == 0 ) {
      te->fm = 0x77;
      te->to = 0x67;
      te->nari = 0;
      te->uti = 0;
      return 1;
    } else if ( bo->mi.count == 2 ) {
      te->fm = 0x96;
      te->to = 0x87;
      te->nari = 0;
      te->uti = 0;
      return 1;
    }
  } else {
    if ( bo->mi.count == 1 ) {
      te->fm = 0x14;
      te->to = 0x23;
      te->nari = 0;
      te->uti = 0;
      return 1;
    } else if ( bo->mi.count == 3 ) {
      te->fm = 0x33;
      te->to = 0x43;
      te->nari = 0;
      te->uti = 0;
      return 1;
    }
  }
   
  return 0;
}
